require 'qrencode.so'
begin
  require 'RMagick'
rescue LoadError
  require 'rubygems'
  require 'RMagick'
end

class QRcode
  def empty_row(margin = 4)
    Array.new(margin + width + margin, false)
  end

  def each_row(margin = 4)
    block_given? or return to_enum(__method__, margin)

    if 0 < margin
      margin.times {
        yield empty_row(margin)
      }

      width.times { |y|
        row = empty_row(margin)
        row[margin...-margin] = (0...width).map { |x| self[x, y] }
        yield row
      }

      margin.times {
        yield empty_row(margin)
      }
    else
      width.times { |y|
        row = (0...width).map { |x| self[x, y] }
        yield row
      }
    end

    self
  end

  def each(margin = 4)
    block_given? or return to_enum(__method__, margin)

    each_row(margin).with_index { |row, y|
      row.each.with_index { |filled, x|
        yield [x, y, filled]
      }
    }

    self
  end

  include Enumerable

  def draw_magick(options = {})
    margin           = options[:margin] || 4
    cell_size        = options[:cell_size] || 2
    foreground_color = options[:foreground_color] || '#000000'
    background_color = options[:background_color] || '#ffffff'

    size = (margin + width + margin) * cell_size

    image = Magick::Image.new(size, size) {
      self.background_color = background_color
    }
    gc = Magick::Draw.new
    gc.fill(foreground_color)

    each(margin) { |x, y, filled|
      if filled
        left = x * cell_size
        top  = y * cell_size
        gc.rectangle(left, top, left + cell_size - 1, top + cell_size - 1)
      end
    }

    gc.draw(image)
    image
  end
end
